/*

Copyright (c) 2001 Michael Terry

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "xpad.h"
#include "fio.h"
#include "color.h"
#include "font.h"
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>

/* static data */
char working_dir[1024];
const char *WINDOW_FILE = "windows";
pad_info DEFAULT_INFO = {0, 0, 260, 260, 0xe000, 0xe000, 0x5600, "-bitstream-charter-medium-r-normal-*-*-120-*-*-p-*-iso8859-1", ""};
pad_node *first_pad = NULL;
pad_node *last_pad = NULL;


pad_node *get_pad (GtkWidget *window)
{
	pad_node *current = first_pad;

	while (current != NULL)
	{
		if (current->window == window)
			return current;

		current = current->next;
	}

	return NULL;
}

void free_all_pads ()
{
	pad_node *current = first_pad;

	while (first_pad != NULL)
	{
		current = first_pad;
		first_pad = first_pad->next;

		gtk_widget_destroy (current->window);
		free (current);
	}
}

void quit_if_no_pads ()
{
	gboolean pad_exists = FALSE;
	pad_node *current = first_pad;

	if (current == NULL)
		gtk_main_quit();

	while (current != NULL)
	{
		if (current->hidden == FALSE)
		{
			pad_exists = TRUE;
			break;
		}

		current = current->next;
	}

	if (pad_exists == FALSE)
	{
		free_all_pads();
		gtk_main_quit();
	}
}

/* destroy one pad */
void pad_destroy (GtkWidget *window)
{
	pad_node *current = first_pad;
	pad_node *old_node = first_pad;
	
	if (window == NULL || first_pad == NULL) 
		return;

	/* first, find window in linked list, and remove pad */
	if (first_pad->window == window) /* first in list */
		first_pad = first_pad->next;
	else
	{
		while (current->next != NULL)
		{
			if (current->next->window == window)
			{
				printf("got it\n");
				old_node = current->next;
				current->next = current->next->next;
				
				if (old_node == last_pad)
					last_pad = current;
				
				printf("freed it\n");
				break;
			}
			current = current->next;
		}
	}
	
	/* then, destroy the widget */
	gtk_widget_destroy (window);

	free(old_node);
	
	save_pads ();

	quit_if_no_pads ();
}

void pad_close (GtkWidget *window)
{
	if (window == NULL || first_pad == NULL) 
		return;

	/* then, destroy the widget */
	gtk_widget_hide(window);

	(get_pad (window))->hidden = TRUE;
	
	save_pads();

	quit_if_no_pads ();
}



/* text loses focus, so we make it uneditable */
/* assumes widget is a GtkText */
void text_loses_edit (GtkWidget *widget, GdkEvent *event, gpointer gp)
{
	gtk_text_set_editable (GTK_TEXT (widget), FALSE);
	save_pads();
//	gdk_window_lower ( gtk_widget_get_parent_window (widget) );
}



/* text gains focus, so we make it editable */
/* assumes widget is a GtkText */
void text_set_editable (GtkWidget *text, GtkWidget *menu_item)
{
	gtk_text_set_editable (GTK_TEXT (text), TRUE);
        gtk_widget_grab_focus (text);
        gdk_window_raise ( gtk_widget_get_parent_window (text) );
}



void pad_resized ()
{
	save_pads();
}



void detacher_stub (GtkWidget *widget, GtkMenu *menu)
{
 
  
}  


static gboolean popup_handler (GtkWidget *textbox, GdkEvent *event, gpointer user_data)
{
	GtkMenu *menu;
	GdkEventButton *event_button;

	g_return_val_if_fail (user_data != NULL, FALSE);
	g_return_val_if_fail (GTK_IS_MENU (user_data), FALSE);
	g_return_val_if_fail (event != NULL, FALSE);
	
	menu = GTK_MENU (user_data);

	if (event->type == GDK_BUTTON_PRESS)
	{
		event_button = (GdkEventButton *) event;
		if (event_button->button == 3)
		{
			gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event_button->button, event_button->time);
       			return TRUE;
		}
		else if (event_button->button == 1)
		{
/*			gtk_widget_set_sensitive (textbox, FALSE);
			return TRUE;
*/
		}
	}
	
	return FALSE;
}

/*
void mouse_release (GtkWidget *textbox, GdkEvent *event, gpointer user_data)
{
	GdkEventButton *event_button;

	if (event->type == GDK_BUTTON_PRESS)
	{
		event_button = (GdkEventButton *) event;
		if (event_button->button == 3)
		{
			gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event_button->button, event_button->time);
       			return TRUE;
		}
		else if (event_button->button == 1)
		{
			gtk_widget_set_sensitive (textbox, FALSE);
			return TRUE;

		}
	}
	
	return FALSE;
}
*/

GtkWidget *create_pad(pad_info *info)
{
	GtkWidget *window;
	GtkWidget *textbox;
	GtkWidget *menu;
	GtkWidget *menu_item_font;
	GtkWidget *menu_item_edit;
	GtkWidget *menu_item_new_pad;
	GtkWidget *menu_item_destroy;
	GtkWidget *menu_item_close;
	GtkWidget *menu_item_color;
	GdkColor color;
	GtkStyle *style;
	pad_node *node = (pad_node *) malloc(sizeof(pad_node));	

	GtkWidget *root = gtk_menu_item_new_with_label ("root");
	gtk_widget_show (root);

	if (info == NULL)
		info = &DEFAULT_INFO;
	
	/* make widgets */
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size (GTK_WINDOW (window), info->width, info->height);
	textbox = gtk_text_new (NULL, NULL);
	menu = gtk_menu_new ();
	menu_item_edit = gtk_menu_item_new_with_label ("Edit");
	menu_item_new_pad = gtk_menu_item_new_with_label ("New Pad");
	menu_item_destroy = gtk_menu_item_new_with_label ("Destroy");
	menu_item_close = gtk_menu_item_new_with_label ("Close");
	menu_item_color = gtk_menu_item_new_with_label ("Color");
	menu_item_font = gtk_menu_item_new_with_label ("Font");
	gtk_menu_append (GTK_MENU (menu), menu_item_edit);
	gtk_menu_append (GTK_MENU (menu), menu_item_color);
	gtk_menu_append (GTK_MENU (menu), menu_item_font);
	gtk_menu_append (GTK_MENU (menu), menu_item_new_pad);
	gtk_menu_append (GTK_MENU (menu), menu_item_close);
	gtk_menu_append (GTK_MENU (menu), menu_item_destroy);
	
	/* if using default, center */
	if (info == &DEFAULT_INFO)
		gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	else
		gtk_widget_set_uposition (window, info->x, info->y);


	/* set content of textbox */
	gtk_text_insert (GTK_TEXT (textbox), NULL, NULL, NULL, info->content, -1);

	/* set color of textbox */
	style = gtk_style_new();
	color.red = info->back_red;
	color.green = info->back_green;
	color.blue = info->back_blue;
	style->base[GTK_STATE_NORMAL] = color;
	style->font = gdk_font_load(info->fontname);
	strcpy (node->fontname, info->fontname);
	gtk_widget_set_style (textbox, style);

	/* set textbox's properties */
	gtk_text_set_editable (GTK_TEXT (textbox), TRUE);
	gtk_text_set_word_wrap (GTK_TEXT (textbox), TRUE);
	gtk_text_set_line_wrap (GTK_TEXT (textbox), TRUE);


	/* add textbox to window */
	gtk_container_add (GTK_CONTAINER (window), textbox);
	
	/* if a wm wants us to close, we close */
//	gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (pad_close), (gpointer) window);

	gtk_signal_connect (GTK_OBJECT (textbox), "focus-out-event", GTK_SIGNAL_FUNC (text_loses_edit), NULL);
	gtk_signal_connect (GTK_OBJECT (textbox), "event", GTK_SIGNAL_FUNC (popup_handler), GTK_WIDGET (menu));
/*	gtk_signal_connect (GTK_OBJECT (textbox), "button-release-event", GTK_SIGNAL_FUNC (mouse_release), NULL);
	gtk_signal_connect (GTK_OBJECT (window), "size-allocate", GTK_SIGNAL_FUNC (pad_resized), NULL);
*/	
	gtk_signal_connect_object (GTK_OBJECT (menu_item_destroy), "activate", GTK_SIGNAL_FUNC (pad_destroy), (gpointer) window);
	gtk_signal_connect_object (GTK_OBJECT (menu_item_close), "activate", GTK_SIGNAL_FUNC (pad_close), (gpointer) window);
        gtk_signal_connect_object (GTK_OBJECT (menu_item_edit), "activate", GTK_SIGNAL_FUNC (text_set_editable), (gpointer) textbox);
	gtk_signal_connect_object (GTK_OBJECT (menu_item_new_pad), "activate", GTK_SIGNAL_FUNC (create_pad), NULL);
	gtk_signal_connect_object (GTK_OBJECT (menu_item_color), "activate", GTK_SIGNAL_FUNC (color_select), (gpointer) textbox);
	gtk_signal_connect_object (GTK_OBJECT (menu_item_font), "activate", GTK_SIGNAL_FUNC (font_select), (gpointer) node);

	/* display widgets */	
	gtk_widget_show (textbox);
	gtk_widget_show (menu_item_edit);
	gtk_widget_show (menu_item_new_pad);
	gtk_widget_show (menu_item_destroy);
	gtk_widget_show (menu_item_color);
	gtk_widget_show (menu_item_font);
	gtk_widget_show (menu_item_close);
	gtk_widget_realize (window); /* still have to set decorations */

	gtk_menu_item_set_submenu (GTK_MENU_ITEM (root), menu);
	
	node->hidden = FALSE;
	node->next = NULL;
	node->window = window;
	
	/* check if this is first pad made */
	if (first_pad == NULL)
	{
		last_pad = node;
		first_pad = node;
	}
	else
	{
		last_pad->next = node;
		last_pad = node;
	}

	/* remove wm decorations */
        gdk_window_set_decorations (window->window, 0);

        gtk_widget_show (window);

	return window;
}


int main (int argc, char *argv[])
{
	gtk_set_locale ();
	gtk_init(&argc, &argv);

	strcpy (working_dir, getenv("HOME"));
	strcat (working_dir, "/.xpad/");

	/* make all needed files */
	mkdir (working_dir, 00755);

	/* load all pads */
	load_pads();

	gtk_main ();
	gtk_exit (0);
	
	return 0;
}














